//
// Copyright 2002-2015, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.
//
// Warning:  not all files in this directory structure may be covered by the
// same copyright.  Some of them may be part of the GNU source distribution
// and you must obey the GPL copyright for any such files.

#include "BrashCommandLine.h"
#include "BrashRedirection.h"
#include <cxxtls/stringexpressionparser.h>

using namespace std;
using namespace cxxtls;


BrashCommandLine::
BrashCommandLine(string::const_iterator &f,
                 string::const_iterator const &l
                )
{
    // this is an obsolete debug time only helper tool

    int wordCount=0;

    arguments_.reset    (  new ARGUMENTS_LIST()  );
    redirections_.reset (  new REDIRECTION_LIST() );

    while(f != l)
    { // parse command line tokens

        StringExpr::skipSeparators(f,l);

        while(f !=l)
        { // check for a redirections

           char c = *f;
           
           bool append = false;

           int fileNo=1;
           int otherFile=-1;  // -1 means not used

           bool numberedRedirection = RedirType::isNumberedRedirection(f, l, fileNo);

           if (f != l)
              c = *f;

           if(c == '>' || c == '<')
           {  // redirection found


              ++f;

              if(f ==l)
                 --f;
              else
              if(*f != c)
                 --f;
              else
                 append = true;

              // if we get here, we have a redirection

              ++f;  // move past c

              if(f == l)
              {
                 throw InvalidRedirection();
              }

              if(*f == '&')
              {
                 // handle "otherFile" duplications like:      2>&1
                 // which means that stderr is the same as file number stdout.

                 ++f;

                 if(f == l || !isdigit(*f))
                     throw InvalidRedirection();

                 otherFile = 0;

                 while(f != l && isdigit(*f))
                 {
                    otherFile *= 10;
                    char c = *f++;

                    otherFile += c - '0';
                 }



              }

              bool input=false;
                  
              if(c == '<')
                 input = true;


              if(otherFile != -1)
              {
                   redirections_->push_back(REDIRECTION_LIST::value_type(RedirType(fileNo, input, append, otherFile), (cxxtls::StringExpression *)0) );
              }
              else
              {
                  StringExpr::skipSeparators(f,l);
                  
                  if(f == l)
                  {
                     throw InvalidRedirection();
                  }
                  
                  
                  auto_ptr<StringExpression> e(StringExpr::parse(f,l));
                  
                  if(!e.get())
                  {
                     throw InvalidRedirection();
                  }
                  
                  
                  
                  
                  if(!numberedRedirection)
                  {
                      if(input)
                         fileNo = 0;  
                  }
                  
                  {
                      // keep these statements together for exception safety.
                      redirections_->push_back(REDIRECTION_LIST::value_type(RedirType(fileNo, input, append, otherFile), e.get()) );
                      e.release();
                  }
                  
              }

              StringExpr::skipSeparators(f,l);


           }  // redirection found
           else
              break; // no redir found

        } // check for redirection operators

        if(f == l )
           break;    // might have had a redirection at the end of the line


        StringExpr::skipSeparators(f,l);

        if(f != l)
        {
            char c = *f;

            if(c == '&' || c == ';' || c == '|' || c == '\n')
               break;  // this an end of command terminator
            
        }

        auto_ptr<StringExpression> e(StringExpr::parse(f,l));

        if(!e.get())
        {
           throw InvalidWordSyntaxException();
        }

        ++wordCount;

        if(wordCount >= 65000)
        {
           throw TooLongException();
        }

        // keep the following 2 lines together, they must not be separated for
        // exception saftey.
        {
            arguments_->push_back(e.get());
            e.release();
        }

    } // parse command line tokens

    if(arguments_->empty())
    {
       throw EmptyCommandException();
    }



}


std::ostream &operator<<( std::ostream &s, BrashCommandLine const &me)
{
    if(!me.arguments_.get())
    {
       return s << "BrashCommandLine(EMPTY)";
    }

    std::string name;

    (*me.arguments_->begin())->isSimple(name);

    return s << "BrashCommandLine(" << name << ")";
}
