//
// Copyright 2002-2015, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.
//
// Warning:  not all files in this directory structure may be covered by the
// same copyright.  Some of them may be part of the GNU source distribution
// and you must obey the GPL copyright for any such files.
//
// This module is included to suppress NT's annoying behaviour of
// popping up an 'unhandled exception' dialog box whenever a segment
// violation occurs -- or other special errors that occur only on nt.
//
// When used, this object module MUST BE FIRST in your link order!
//

///@file
///
/// This file defines an NT specific operating system exception
/// handler.  Basically, it catches unexpected exceptions produced
/// by the operating system calls, as opposed to normal C++ exceptions.
/// 
/// These exceptions are converted into calls to exit() with an explanation
/// of what happened so that you can see that an exception occurred.  Strangely,
/// this not the default behavior on NT....


#ifndef _MSC_VER
namespace IGNORE_THIS_FREAKING_NAMESPACE_DARNIT 
{
  static int trashola=1;  // if not on NT, ignore this object module
  void doit()
  {
    trashola=2;
  }
};
#else

// This stuff superseeds the default OS exception handler -- when exceptions
// occur, the program dies WITHOUT putting up a dialog box.  This must be
// the first object module linked into your executable to ensure that static
// initializations that cause errors do not pop up annoying dialog boxes that
// hang build scripts -- instead they merely crash producing error messages.
//
// Note that if this module is not linked first, it will still work -- but
// will not protect object modules linked earlier!  It really should be first.


#include <windows.h>
#include <portable_io.h>
#include <stdlib.h>
                
#include "tchar.h"
#include "stdio.h"
#include "psapi.h"
#include <string>

// Important: Must include psapi.lib in additional dependencies section
// In VS2005... Project > Project Properties > Configuration Properties > Linker > Input > Additional Dependencies

std::string getCurrentProcessName()
{
    std::string rv;

    HANDLE Handle = OpenProcess(
        PROCESS_QUERY_INFORMATION | PROCESS_VM_READ,
        FALSE,
        GetCurrentProcessId()
    );

    if (Handle) 
    {
        TCHAR Buffer[MAX_PATH];
        if (GetModuleFileNameEx(Handle, 0, Buffer, MAX_PATH))
        {
            // At this point, buffer contains the full path to the executable

            for(int i = 0; i < MAX_PATH; ++i)
               rv += char(Buffer[i]);
        }
        else
        {
            // You better call GetLastError() here
        }
        CloseHandle(Handle);
    }
    
    return rv;

}

namespace
   /// private namespace that wraps the exception handler code
   /// up so that we don't get namespace collisions.
{
  LONG WINAPI death_by_exception(_EXCEPTION_POINTERS *ExceptionInfo)
  {
    /// Catch the system exceptions and exit the program with
    /// a message to stdout.


    fprintf(stderr, "\n\n\nSegfault or other major exception has occurred in process:\n    %s\n\n\n",
            getCurrentProcessName().c_str()
           );
    fflush(stderr);
    exit(1);
    return 0;
  }
  
  struct ForceStaticInvocation
    /// A hook class whose purpose is to be constructed at
    /// at static init time and thus install the exception handler.
  {
    ForceStaticInvocation() /// and install the exception handler.
    {
      SetUnhandledExceptionFilter(death_by_exception);
    }
  };
  
  ForceStaticInvocation exceptionHandlerInstaller; ///< constructed to install the handler
};
#endif

/*
  This is the test program for the above

int main()
{
  double *d = (double*)0x1;

  *d = 1.0;
}

*/

